#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'hardware_information/class/obtain.rb'
# =========================================================================== #
class HardwareInformation

  require 'hardware_information/monitor/monitor.rb'
  # ========================================================================= #
  # === obtain_information_about_the_monitor
  # ========================================================================= #
  def obtain_information_about_the_monitor
    make_line(
      '  Monitor resolution',
      ::HardwareInformation.return_the_monitor_resolution
    ) { :mediumspringgreen }
  end

  # ========================================================================= #
  # === obtain_information_about_the_GPU
  # ========================================================================= #
  def obtain_information_about_the_GPU
    make_line(
      '  GPU',
      return_information_about_the_GPU
    ) { :mediumpurple }
  end

  # ========================================================================= #
  # === return_information_about_the_GPU
  #
  # This method will return information about the GPU in use on the given
  # computer system.
  #
  # This may be similar to a String such as:
  #
  #   GPU: AMD ATI Radeon R7 Graphics
  #
  # ========================================================================= #
  def return_information_about_the_GPU
    result = `lspci #{ERROR_LINE}`.strip
    if result and result.include?('VGA compatible controller')
      result = result.split(N).select {|line|
        line.include? 'VGA compatible controller'
      }.first
      # ===================================================================== #
      # Work on:
      #
      #   VGA compatible controller: Advanced Micro Devices, Inc. [AMD/ATI] Kaveri [Radeon R7 Graphics]"
      #
      # ===================================================================== #
      result = result.split('. [').last.
               delete('[]').tr('/',' ')
    end
    return result
  end

  # ========================================================================= #
  # === obtain_firmware_information
  #
  # Note that this functionality depends on the binary called "lshw".
  # ========================================================================= #
  def obtain_firmware_information
    result = `lshw 2>&1`
    if result
      # ===================================================================== #
      # See: https://rubular.com/r/Uf4XqmbZW3atCy
      # ===================================================================== #
      regex = 
        /description: Motherboard\s+product: (.+)\s+vendor: (.+)\s+[.:0-9a-zA-Z\n ]+\*-firmware[.:0-9a-zA-Z\n ]+version: (\d{0,4}[0-9.]+)\s+date: (\d{1,2}.+)/
      result =~ regex
      motherboard = $1.to_s.dup
      motherboard_vendor = $2.to_s.dup
      bios_version = $3.to_s.dup
      bios_release = $4.to_s.dup
      # ===================================================================== #
      # Show some more information next, in particular the BIOS.
      # ===================================================================== #
      make_line('  Motherboard vendor', motherboard_vendor)
      make_line('  Motherboard', motherboard)
      make_line('  BIOS version', bios_version)
      make_line('  BIOS released on', bios_release)
    end
  end

  # ========================================================================= #
  # === obtain_vendor_id
  # ========================================================================= #
  def obtain_vendor_id
    result = LSCPU.grep(/Vendor ID:/).first.split(':').last.strip
    make_line('  Vendor ID', result) { :mediumseagreen }
  end

  # ========================================================================= #
  # === obtain_n_cores
  #
  # Return how many cores that machine has.
  # ========================================================================= #
  def obtain_n_cores
    make_line(
      '  n CPU cores',
      `nproc --all`.strip
    ) { :lightgreen }
  end

  # ========================================================================= #
  # === obtain_ram                                                  (ram tag)
  # ========================================================================= #
  def obtain_ram
    make_line('  RAM (max)', RAM_AVAILABLE) { :orangered }
  end

  # ========================================================================= #
  # === obtain_ram_free
  # ========================================================================= #
  def obtain_ram_free
    make_line('  RAM (free)', RAM_AVAILABLE_FREE) { :orangered }
  end

  # ========================================================================= #
  # === obtain_swap_size                                           (swap tag)
  # ========================================================================= #
  def obtain_swap_size
    _ = DATASET_FROM_PROC_MEMINFO.grep(/SwapTotal:/).first.
        split('  ').last.strip.upcase
    make_line('  SwapTotal', _) { :springgreen }
  end

  # ========================================================================= #
  # === obtain_system_bogomips
  #
  # BogoMIPS measure CPU speed.
  # ========================================================================= #
  def obtain_system_bogomips
    make_line '  BogoMIPS',
      SPLITTED_CPUINFO.select {|line|
        line.include? 'bogomips'
      }.first.split(':')[1].strip
  end

  # ========================================================================= #
  # === obtain_pci_information
  # ========================================================================= #
  def obtain_pci_information
    array_fetch_these_types = %w(
      SMBus Audio IDE VGA
      Memory Controller
    )
    results = []
    if LSPCI
      LSPCI.split(N).each {|line|
        array_fetch_these_types.each { |inner_line|
          results << line if line.include? inner_line
        }
      }
      make_line '  PCI Devices', results
    end
  end

  # ========================================================================= #
  # === obtain_chip_mhz
  # ========================================================================= #
  def obtain_chip_mhz
    make_line '  Chip MHzx',
      SPLITTED_CPUINFO.select {|line|
        line.include? 'cpu MHz'
      }.first.split(':')[1].strip
  end

  # ========================================================================= #
  # === obtain_cache_size
  # ========================================================================= #
  def obtain_cache_size
    make_line '  Cache Size',
      SPLITTED_CPUINFO.select {|line|
        line.include? 'cache size'
      }.first.split(':')[1].strip
  end

  # ========================================================================= #
  # === obtain_cpu_model
  # ========================================================================= #
  def obtain_cpu_model
    result = SPLITTED_CPUINFO.select {|line|
      line.include? 'model name'
    }.first.split(':')[1].strip
    make_line '  Model', result
  end

  # ========================================================================= #
  # === obtain_byte_order
  # ========================================================================= #
  def obtain_byte_order
    result = LSCPU.grep(/Byte Order:/).first.split(':').last.strip
    make_line '  Byte Order', result
  end

  # ========================================================================= #
  # === obtain_cpu_family
  # ========================================================================= #
  def obtain_cpu_family
    result = LSCPU.grep(/CPU family:/).first.split(':').last.strip
    make_line '  CPU family', result
  end

  # ========================================================================= #
  # === obtain_model
  # ========================================================================= #
  def obtain_model
    result = LSCPU.grep(/Model:/).first.split(':').last.strip
    make_line '  Model', result
  end

  # ========================================================================= #
  # === obtain_virtualization
  # ========================================================================= #
  def obtain_virtualization
    result = LSCPU.grep(/Virtualization:/).first.split(':').last.strip
    make_line('  Virtualization', result) { :mediumseagreen }
  end

  # ========================================================================= #
  # === obtain_cpu_op_modes
  # ========================================================================= #
  def obtain_cpu_op_modes
    result = LSCPU.grep(/CPU op-mode/).first.split(':').last.strip
    make_line('  CPU op-mode(s)', result)
  end

  # ========================================================================= #
  # === obtain_linux_kernel_version
  #
  # This method should return a String, such as '4.14.4'.
  # ========================================================================= #
  def obtain_linux_kernel_version
    result = `uname -r`.strip
    make_line('  Linux kernel version', result)
  end

  # ========================================================================= #
  # === obtain_architecture
  # ========================================================================= #
  def obtain_architecture
    result = LSCPU.grep(/Architecture:/).first.split(':').last.strip
    make_line('  Architecture', result) { :violet }
  end

end